library(ncdf4)
#library(ggplot2)
#library(lubridate)

rm(list=ls()) ; gc()

#Directory information
setwd("../../../scratch/")

################################################################################
#Load ERA5 data
message("Loading and preprocessing data...")
nc <- nc_open("g500_5922.nc")
nc_lon <- ncvar_get(nc,"lon")
nc_lat <- ncvar_get(nc,"lat")
nc_var <- ncvar_get(nc,"g500") #x,y,time

#Selection of spatial domains
gridsize_x <- abs(nc_lon[2]-nc_lon[1])
gridsize_y <- abs(nc_lat[2]-nc_lat[1])

#Small domain
lon <- which(nc_lon %in% seq(-15,15,by=gridsize_x))
lat <- which(nc_lat %in% seq(32,48,by=gridsize_y))
nc_var <- nc_var[lon,lat,]

#Organize and Standardize data
var <- matrix(nc_var,dim(nc_var)[1]*dim(nc_var)[2],dim(nc_var)[3]) #space,time
rm(nc_var) ; gc()

message("Standardizing...")
var.sc <- scale(t(var),center=T,scale=T) #var[,1:365]
message("Done.")
################################################################################
#Data reduction
message("Data reduction by means of EOFs...")
RR=cor(var.sc)
DD=svd(RR)

U=DD$u ; V=DD$v
dv=DD$d ; dd=diag(DD$d)

FF=U%*%dd^0.5 #loadings matrix
A=dv^(-0.5)
AA=diag(A)
Zs=var.sc%*%U%*%AA  #CPs matrix

SumDiag=sum(dv)
VAR=(dv/SumDiag)*100 #explained variance

k=which(cumsum(VAR)>90)[1]
rm(RR,DD,U,V,A,AA,dd,FF) ; gc()
message("Done.")
################################################################################
#Clustering procedure
message("Performing clustering...")
input.data <- Zs[,1:k]
ward_clustering <- hclust(d = dist(input.data),method = "ward.D2")

#Sensibility to the number of clusters
wss <- (nrow(input.data)-1)*sum(apply(input.data,2,var))
for(i in 2:15){
wss.sub <- NA
  for(j in 1:i){
  sub.data <- input.data[which(cutree(ward_clustering,k=i)==j),]
  wss.sub[j] <- (nrow(sub.data)-1)*sum(apply(sub.data,2,var))
  }
  wss[i] <- sum(wss.sub)
}

png.save <- function(x,filename,main,xlab,ylab){
  png(filename = paste(filename,".png",sep=""))
  plot(x,type="o",main=main,xlab = xlab,ylab=ylab)
  dev.off()
}

png.save(wss,filename = "wss",main="Ward g500 RedDomain 1959-2022",
         xlab="Number of patterns",ylab="Total within sum of squares")
message("Done.")
